==> symbol-processing-aa-embeddable-2.3.0-sources.jar: org/jetbrains/kotlin/fir/analysis/checkers/expression/FirInlineBodyResolvedQualifierChecker.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.checkers.expression

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.fir.FirElement
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.isExplicitParentOfResolvedQualifier
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.type.FirInlineExposedLessVisibleTypeChecker
import ksp.org.jetbrains.kotlin.fir.declarations.fullyExpandedClass
import ksp.org.jetbrains.kotlin.fir.declarations.utils.isCompanion
import ksp.org.jetbrains.kotlin.fir.declarations.utils.isStatic
import ksp.org.jetbrains.kotlin.fir.expressions.FirQualifiedAccessExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirResolvedQualifier
import ksp.org.jetbrains.kotlin.fir.expressions.toResolvedCallableSymbol
import ksp.org.jetbrains.kotlin.fir.resolve.defaultType
import ksp.org.jetbrains.kotlin.utils.addToStdlib.applyIf

object FirInlineBodyResolvedQualifierChecker : FirResolvedQualifierChecker(MppCheckerKind.Common) {
    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(expression: FirResolvedQualifier) {
        val inlineFunctionBodyContext = context.inlineFunctionBodyContext ?: return
        if (expression.isExplicitParentOfResolvedQualifier()) return
        val accessedClass = expression.symbol
            ?.applyIf(expression.resolvedToCompanionObject) {
                fullyExpandedClass()?.resolvedCompanionObjectSymbol
            }
            ?: return
        val source = expression.source ?: return

        if (accessedClass.isCompanion) {
            inlineFunctionBodyContext.checkAccessedDeclaration(
                source, expression, accessedClass,
            )
        }

        val parentElement = context.containingElements.elementAtOrNull(context.containingElements.size - 2)

        if (!expression.isStaticCallReceiver(parentElement)) {
            FirInlineExposedLessVisibleTypeChecker.check(accessedClass.defaultType(), source, inlineFunctionBodyContext)
        }
    }

    /**
     * True when [this] is the explicit receiver of a static call. Static calls with explicit receivers are:
     *
     * - Constructor calls of non-inner nested classes
     * - Callable references with non-bound receiver
     * - Static field access or method call
     *
     * We want to suppress diagnostics on these receivers because they're always duplicated by a diagnotic on the call itself.
     */
    private fun FirResolvedQualifier.isStaticCallReceiver(parentElement: FirElement?): Boolean {
        return parentElement is FirQualifiedAccessExpression
                && parentElement.explicitReceiver == this
                && parentElement.extensionReceiver != this
                && (parentElement.dispatchReceiver != this || parentElement.toResolvedCallableSymbol()?.isStatic == true)
    }
}

==> symbol-processing-aa-embeddable-2.3.4-sources.jar: org/jetbrains/kotlin/fir/analysis/checkers/expression/FirInlineBodyResolvedQualifierChecker.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.checkers.expression

import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.fir.FirElement
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.isExplicitParentOfResolvedQualifier
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.type.FirInlineExposedLessVisibleTypeChecker
import ksp.org.jetbrains.kotlin.fir.declarations.fullyExpandedClass
import ksp.org.jetbrains.kotlin.fir.declarations.utils.isCompanion
import ksp.org.jetbrains.kotlin.fir.declarations.utils.isStatic
import ksp.org.jetbrains.kotlin.fir.expressions.FirQualifiedAccessExpression
import ksp.org.jetbrains.kotlin.fir.expressions.FirResolvedQualifier
import ksp.org.jetbrains.kotlin.fir.expressions.toResolvedCallableSymbol
import ksp.org.jetbrains.kotlin.fir.resolve.defaultType
import ksp.org.jetbrains.kotlin.utils.addToStdlib.applyIf

object FirInlineBodyResolvedQualifierChecker : FirResolvedQualifierChecker(MppCheckerKind.Common) {
    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(expression: FirResolvedQualifier) {
        val inlineFunctionBodyContext = context.inlineFunctionBodyContext ?: return
        if (expression.isExplicitParentOfResolvedQualifier()) return
        val accessedClass = expression.symbol
            ?.applyIf(expression.resolvedToCompanionObject) {
                fullyExpandedClass(context.session)?.resolvedCompanionObjectSymbol
            }
            ?: return
        val source = expression.source ?: return

        if (accessedClass.isCompanion) {
            inlineFunctionBodyContext.checkAccessedDeclaration(
                source, expression, accessedClass,
            )
        }

        val parentElement = context.containingElements.elementAtOrNull(context.containingElements.size - 2)

        if (!expression.isStaticCallReceiver(parentElement)) {
            FirInlineExposedLessVisibleTypeChecker.check(accessedClass.defaultType(), source, inlineFunctionBodyContext)
        }
    }

    /**
     * True when [this] is the explicit receiver of a static call. Static calls with explicit receivers are:
     *
     * - Constructor calls of non-inner nested classes
     * - Callable references with non-bound receiver
     * - Static field access or method call
     *
     * We want to suppress diagnostics on these receivers because they're always duplicated by a diagnotic on the call itself.
     */
    private fun FirResolvedQualifier.isStaticCallReceiver(parentElement: FirElement?): Boolean {
        return parentElement is FirQualifiedAccessExpression
                && parentElement.explicitReceiver == this
                && parentElement.extensionReceiver != this
                && (parentElement.dispatchReceiver != this || parentElement.toResolvedCallableSymbol()?.isStatic == true)
    }
}
